﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace NTL.KinectLib
{
    public class CharpAudio
    {
        private double K = Math.Pow(2.0, (1.0 / 12.0)); //【１オクターブ(倍音)を12等分割】
        private double Fb = 440.0; //【基準音はラ(A4)】440Hz
        private int nF = 85;       //【A1～A8】55Hz～7040Hz この音をminからmaxまでに割り当てる

        public double[] Frequency; //【音階周波数】
        private const int nSineWave = 10000; //【１周期10000サンプル】
        public double[] SineWave = new double[nSineWave];  //【サイン波】
        private const int nCh = 2;     //【チャンネル】2 ch
        private const int nBits = 16;  //【データ精度】16bits
        private const int SamplingRate = 44100;
        private const int BlockSize = nCh * (nBits >> 3);
        private const int DataTransferRate = SamplingRate * BlockSize;
        private const int RecordSeconds = 2;
        private const int DataLength = DataTransferRate * RecordSeconds;
        private const int HeaderLength = 44;
        private const int FileLength = HeaderLength + DataLength;
        private const int Height = 500; //画面は本来480画素だが20画素分のブランクを設ける
        private const int SamplePerPixel = SamplingRate * RecordSeconds / Height; //176サンプル

        private byte[] header = new byte[] {
            0x52, 0x49, 0x46, 0x46, // 0: "R", "I", "F", "F"
            0x44, 0x62, 0x05, 0x00, // 4: FileSize-8 = 44 - 8 + 44100*4*2 = 352836 = 56244H
            0x57, 0x41, 0x56, 0x45, // 8: "W", "A", "V", "E"
            0x66, 0x6D, 0x74, 0x20, //12: "f", "m", "t", " "
            0x10, 0x00, 0x00, 0x00, //16: "fmt "領域のバイトサイズ 16
            0x01, 0x00,             //20: ID 1:PCM
            0x02, 0x00,             //22: チャンネル数 1:モノラル  2:ステレオ
            0x44, 0xAC, 0x00, 0x00, //24: サンプリング速度 8000:8kSPS  44100=AC44H:44.1kSPS
            0x10, 0xB1, 0x02, 0x00, //28: データ転送速度＝サンプリング速度×ブロックサイズ(byte/秒)
            0x04, 0x00,             //32: チャンネル数×(ビット精度÷8)バイト
            0x10, 0x00,             //34: ビット精度 8:8bit   16:16bit
            0x64, 0x61, 0x74, 0x61, //36: "d", "a", "t", "a"
            0x20, 0x62, 0x05, 0x00  //40: "data"領域のバイトサイズ 2秒分
        }; //【Waveファイルのヘッダ部】
        private byte[] wavedata;   //【Waveファイルのデータ部】

        //******************************************************************************
        //【Initialize】初期化
        //******************************************************************************
        public void Initialize()
        {
            InitializeFrequencyData(); //【音階周波数データを作成する】
            InitializeDirectDigitalSynthesizer(); //【DDSを初期化する】
        }

        //******************************************************************************
        //●【InitializeFrequencyData】音階周波数データを作成する
        //******************************************************************************
        public void InitializeFrequencyData()
        {
            Frequency = new double[nF];
            double Freq = Fb / 8.0;
            for (int i = 0; i < nF; i++)
            {
                Frequency[i] = Freq;
                Freq *= K;
            }
        }

        //******************************************************************************
        //●【InitializeDirectDigitalSynthesizer】DDSを初期化する
        //******************************************************************************
        public void InitializeDirectDigitalSynthesizer()
        {
            double K = 2.0 * Math.PI / (double)nSineWave;
            for (int i = 0; i < nSineWave; i++)
            {
                SineWave[i] = Math.Sin((double)i * K);
            }
        }

        //******************************************************************************
        //【CreateWave】DDSを用いて深度情報に応じて音階データを生成する
        //******************************************************************************
        // volume: 0.0～32767.0
        //【A1～A8】55Hz～7040Hz, 85音階, この音をminからmaxまでに割り当てる 
        public void CreateWave(string filename, int[] depth, int volumeL, int volumeR, int min, int max)
        {
            int i, j, L;
            int d;
            double phase = 0.0;
            int wave;
            double delta;
            double K = (double)nSineWave / (double)SamplingRate;
            int p = HeaderLength;
            CreateWaveFileHeader();
            for (i = 0; i < 480; i++)
            {
                d = depth[i];
                if (d < min) { d = min; }
                if (d > max) { d = max; }
                j = (int)((double)((nF - 1) * (d - min)) / (double)(max - min));
                delta = K * Frequency[j];
                for (L = 0; L < SamplePerPixel; L++)
                {
                    if (phase > (double)nSineWave) { phase -= (double)nSineWave; }
                    wave = (int)((double)volumeL * SineWave[(int)phase]);
                    wavedata[p] = (byte)(wave & 255);
                    wavedata[p + 1] = (byte)((wave >> 8) & 255);
                    wave = (int)((double)volumeR * SineWave[(int)phase]);
                    wavedata[p + 2] = (byte)(wave & 255);
                    wavedata[p + 3] = (byte)((wave >> 8) & 255);
                    phase += delta;
                    p += 4;
                }
            }
            System.IO.FileStream fs = new System.IO.FileStream(
                filename,
                System.IO.FileMode.Create,
                System.IO.FileAccess.Write);
            fs.Write(wavedata, 0, wavedata.Length);
            fs.Close();
        }

        //******************************************************************************
        //●【CreateWaveFileHeader】WaveFileHeaderを作成する
        //******************************************************************************
        private void CreateWaveFileHeader()
        {
            int i;
            wavedata = new byte[FileLength];
            for (i = 0; i < header.Length; i++) { wavedata[i] = header[i]; }
            for (i = 0; i < DataLength; i++) { wavedata[i + HeaderLength] = 0; }
        }
    }
}
